import { world, ItemStack, BlockPermutation } from "@minecraft/server";
import { addItemOrSpawn, decrementItemInHand } from '../util/utils'

world.beforeEvents.worldInitialize.subscribe((e) => {
	e.blockComponentRegistry.registerCustomComponent("t:nest_interaction", new TurkeyNest());
	e.blockComponentRegistry.registerCustomComponent("t:turkey_born", new TurkeyBorn());
	e.itemComponentRegistry.registerCustomComponent("t:eat_cooked_turkey", new TurkeyCookedEat());
	e.itemComponentRegistry.registerCustomComponent("t:eat_raw_turkey", new TurkeyRawEat());
});
class TurkeyRawEat {
	onConsume(e) {
		const { source } = e;
		if (Math.random() <= 0.5) source.addEffect("hunger", 600);
	}
}
class TurkeyCookedEat {
	onConsume(e) {
		const { source } = e;
		source.addEffect("slowness", 100);
		source.addEffect("regeneration", 40);
	}
}
class TurkeyBorn {
	onTick(e) {
		const { block, dimension } = e;
		const quantity = parseInt(block.typeId.match(/_(\d+)$/)[1], 10);
		const direction = block.permutation.getState("minecraft:cardinal_direction");
		block.setPermutation(BlockPermutation.resolve(`t:turkey_nest_${quantity - 1}`, { "minecraft:cardinal_direction": direction }));
		dimension.playSound("block.turtle_egg.crack", block.center());
		dimension.spawnEntity("t:turkey<minecraft:turkey_from_egg>", block.center());
	}
}
class TurkeyNest {
	onPlayerInteract(e) {
		const { player, block, dimension } = e;
		const item = player.getComponent("inventory").container.getItem(player.selectedSlotIndex);
		const quantity = parseInt(block.typeId.match(/_(\d+)$/)[1], 10);
		const direction = block.permutation.getState("minecraft:cardinal_direction");
		const updateBlock = (newQuantity) => {
			block.setPermutation(BlockPermutation.resolve(`t:turkey_nest_${newQuantity}`, { "minecraft:cardinal_direction": direction }));
		};
		if (item?.typeId !== "t:turkey_egg" && quantity > 0) {
			updateBlock(quantity - 1);
			dimension.playSound("block.itemframe.remove_item", block.center());
			addItemOrSpawn(player, new ItemStack("t:turkey_egg", 1));
		} else if (item?.typeId === "t:turkey_egg" && quantity < 4) {
			updateBlock(quantity + 1);
			dimension.playSound("block.itemframe.add_item", block.center());
			decrementItemInHand(player);
		}
	}
}
world.afterEvents.dataDrivenEntityTrigger.subscribe((e) => {
	const { entity, eventId } = e;
	if (eventId === 'turkey.lay_egg') {
		const block = entity.dimension.getBlock(entity.location);
		if (block.typeId.includes("t:turkey_nest")) {
			const quantity = parseInt(block.typeId.match(/_(\d+)$/)[1], 10);
			if (quantity < 4) {
				block.setPermutation(BlockPermutation.resolve(`t:turkey_nest_${quantity + 1}`, {"minecraft:cardinal_direction": block.permutation.getState("minecraft:cardinal_direction")}));
				entity.dimension.playSound("mob.chicken.plop", block.center());
			}
		}
	}
});